<?php

namespace App\Http\Controllers\V1\File;

use App\Http\Controllers\Controller;
use App\Models\File;
use App\Models\User;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Storage;

class FileController extends Controller
{
     /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($idUser)
    {
        try{
            $files = File::where("user_id", $idUser)
            ->get();
            return $files;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los archivos"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only('file', 'user_id');
        $validator = Validator::make($data, [
            'file' => 'required|mimes:jpg,jpeg,png,gif,zip,pdf|max:5048',
            'user_id' => 'required|integer|exists:users,id',
        ]);
        //Si falla la validación error.
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            if (($request->file('file'))) {
                $name = $request->file('file')->getClientOriginalName();
                $nameSimple = pathinfo($name, PATHINFO_FILENAME); 
                $extension = $request->file('file')->getClientOriginalExtension();
                $size = $request->file('file')->getSize();
                $url = '';
                $contador = DB::table('files')
                ->where('nombre', 'like', '%' . $nameSimple . '%')
                ->where('type', '=', $extension)
                ->where("user_id", "=", $request->user_id)
                ->count();
                $nameFile = ($contador > 0) ? $nameSimple.'-'.$contador.'.'.$extension : $name;
                //se guarda el archivo
                $userData = User::findOrFail($request->user_id);
                $correoUser = strtolower($userData->email);
                $path = $request->file('file')->store($correoUser.'/files');

                $url = Storage::url($path);
                //Creamos el archivo en la BD
                $archivo = File::create([
                    "nombre" => $nameFile,
                    "path" => $path,
                    "type" => $extension,
                    'size' => $size,
                    "user_id" => $request->user_id,
                ]);
                //Respuesta en caso de que todo vaya bien.
                return response()->json([
                    'id_file' => $archivo->id,
                    'url' => $url
                ], Response::HTTP_OK);
            }
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al guardar el archivo"]]], 400);
        }
    }
     /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\File  $file
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            //Buscamos el archivo a eliminar
            $archivo = File::findOrfail($id);
            if(Storage::exists($archivo->path)){
                Storage::delete($archivo->path);
            }
            // //Eliminamos el archivo
            $archivo->delete();
            //Devolvemos la respuesta
            return response()->json([
                'message' => 'Se elimino el archivo correctamente'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["No existe el archivo"]]], 400);
        }
    }
}
