<?php

namespace App\Http\Controllers\V1\Recompensa;

use App\Http\Controllers\Controller;
use App\Models\Recompensa;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;

class RecompensaController extends Controller
{


    public function getRecompensasActivas()
    {
        try {
            $recompensas = Recompensa::with('files')->where("estatus", true)->get();
            return $recompensas;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las recompensas"]]], 400);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try {
            $recompensas = Recompensa::with('files')->get();
            return $recompensas;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las recompensas"]]], 400);
        }
    }

    /**
     * Obtiene la informacion de un recompensa
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $recompensa = Recompensa::with('files')->where("id", $id)->first();
            return response()->json($recompensa);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar la recompensa"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only('nombre', 'descripcion_premio', 'meta_canje', 'file_id');
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion_premio' => 'required|max:800|string',
            'meta_canje' => 'integer|required',
            'file_id' => 'integer|required',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $recompensa = Recompensa::create([
                "nombre" => $request->nombre,
                "descripcion_premio" => $request->descripcion_premio,
                'meta_canje' => $request->meta_canje,
                'file_id' => $request->file_id
            ]);
            return response()->json([
                'message' => 'La recompensa se creó correctamente',
                'data' => $recompensa
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al crear el producto."]]], 400);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Recompensa  $recompensa
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //Validamos los datos
        $data = $request->only('nombre', 'descripcion_premio', 'meta_canje', 'file_id', 'estatus');
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion_premio' => 'required|max:800|string',
            'meta_canje' => 'integer|required',
            'file_id' => 'integer|required',
            'estatus' => 'required|boolean',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $recompensaFind = Recompensa::findOrfail($id);
            $recompensaFind->update([
                "nombre" => $request->nombre,
                "descripcion_premio" => $request->descripcion_premio,
                "meta_canje" => $request->meta_canje,
                "estatus" => $request->estatus,
                'file_id' => $request->file_id
            ]);
            return response()->json([
                'message' => 'La recompensa se actualizó correctamente',
                'data' => $recompensaFind
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar la recompensa"]]], 400);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Recompensa  $recompensa
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            $recompensaDelete = Recompensa::find($id);
            if (!$recompensaDelete)
                return response()->json(['error' => ['mensaje' => ["La recompensa no existe"]]], 400);
            $recompensaDelete->delete();
            return response()->json([
                'message' => 'Se eliminó la recompensa correctamente'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar la recompensa"]]], 400);
        }
    }
}
