<?php

namespace App\Models;

use App\Traits\SecureDelete;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Evidencia extends Model
{
    use HasFactory;
    use SecureDelete;

    protected $fillable = [
        'fecha_captura',
        'estatus',
        'comentario',
        'user_id',
        'participante_id',
        'file_id',
    ];

    /**
     * obtiene el archivo relacionado a esta insignia
     */
    public function files()
    {
        return $this->belongsTo(File::class, 'file_id', 'id');
    }

    /**
     * obtiene los retos a los que estan relacionado
     */
    public function retos()
    {
        return $this->belongsToMany(Reto::class, 'evidencia_retos');
    }

    /**
     * obtiene el participante relacionado
     */
    public function participante()
    {
        return $this->belongsTo(Participante::class, 'participante_id', 'id');
    }

    /**
     * obtiene el usuario que la creo
     */
    public function usuario()
    {
        return $this->belongsTo(User::class, 'user_id', 'id');
    }

    /**
     * obtiene las registros de evidencias relacionados a el
     */
    public function registros_evidencias()
    {
        return $this->hasMany(EvidenciaRegistro::class);
    }
}
