<?php

namespace App\Models;

use App\Traits\SecureDelete;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Participante extends Model
{
    use HasFactory;
    use SecureDelete;

    protected $fillable = [
        'nombre_tienda',
        'direccion',
        'nombre_completo',
        'localidad',
        'estado',
        'ciudad',
        'ruta',
        'estatus',
        'notificacion',
        'file_id',
        'user_id',
        'identificador'
    ];

    /**
     * obtiene los retos al participante en una relacion muchos a muchos
     */
    public function retos()
    {
        return $this->belongsToMany(Reto::class, 'reto_participantes');
    }

    /**
     * obtiene el archivo relacionado
     */
    public function files()
    {
        return $this->belongsTo(File::class, 'file_id', 'id');
    }

    /**
     * obtiene el usuario de la persona
     */
    public function usuario()
    {
        return $this->belongsTo(User::class, 'user_id', 'id');
    }

    /**
     * obtiene las evidencias relacionadas a el
     */
    public function evidencias()
    {
        return $this->hasMany(Evidencia::class);
    }
}
