<?php

namespace App\Models;

use App\Traits\SecureDelete;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Reto extends Model
{
    use HasFactory;
    use SecureDelete;

    protected $fillable = [
        'nombre',
        'descripcion',
        'tipo_reto',
        "estatus",
        'user_id',
        'producto_id',
        'file_id',
        'tipo_participante'
    ];


    /**
     * obtiene los participante relacionadas al reto en una relacion muchos a muchos
     */
    public function participantes()
    {
        return $this->belongsToMany(Participante::class, 'reto_participantes');
    }

    /**
     * obtiene los niveles del reto relacionados a el
     */
    public function nivel_retos()
    {
        return $this->hasMany(NivelReto::class);
    }

    /**
     * obtiene el archivo que esta relacionado
     */
    public function files()
    {
        return $this->belongsTo(File::class, 'file_id', 'id');
    }

    /**
     * obtiene los productos a los que esta relacionado
     */
    public function productos()
    {
        return $this->belongsToMany(Producto::class, 'reto_productos');
    }

    /**
     * obtiene las evidencias a los que esta relacionado
     */
    public function evidencias_retos()
    {
        return $this->belongsToMany(Evidencia::class, 'evidencia_retos');
    }
}
